@@ -115,6 +115,16 @@ struct iso_write_opts {
      */
     unsigned int rrip_version_1_10 :1;
 
+    /**
+     * Write field PX with file serial number even with RRIP-1.10
+     */
+    unsigned int rrip_1_10_px_ino :1;
+
+    /**
+     * See iso_write_opts_set_hardlinks()
+     */
+    unsigned int hardlinks:1;
+
     /**
      * Write AAIP as extension according to SUSP 1.10 rather than SUSP 1.12.
      * I.e. without announcing it by an ER field and thus without the need
@@ -269,7 +279,9 @@ struct ecma119_image
     unsigned int eltorito :1;
     unsigned int iso1999 :1;
 
-    unsigned int aaip :1;  /* whether to write eventual ACLs and EAs */
+    unsigned int hardlinks:1; /* see iso_write_opts_set_hardlinks() */
+
+    unsigned int aaip :1;     /* see iso_write_opts_set_aaip() */
 
     /* allways write timestamps in GMT */
     unsigned int always_gmt :1;
@@ -291,6 +303,9 @@ struct ecma119_image
     /** Write old fashioned RRIP-1.10 rather than RRIP-1.12 */
     unsigned int rrip_version_1_10 :1;
 
+    /** Write field PX with file serial number even with RRIP-1.10 */
+    unsigned int rrip_1_10_px_ino :1;
+
     /* Write AAIP as extension according to SUSP 1.10 rather than SUSP 1.12. */
     unsigned int aaip_susp_1_10 :1;
 
@@ -318,12 +333,18 @@ struct ecma119_image
      */
     int sort_files;
 
+
+#ifndef Libisofs_hardlink_matcheR
+
+    /* ts A90508 : <<< this is on its way out */
     /**
      * In the CD, each file must have an unique inode number. So each
      * time we add a new file, this is incremented.
      */
     ino_t ino;
 
+#endif /* ! Libisofs_hardlink_matcheR */
+
     char *input_charset;
     char *output_charset;
 
