@@ -8,6 +8,7 @@
  */
 
 #include "libisofs.h"
+#include "image.h"
 #include "node.h"
 #include "stream.h"
 #include "aaip_0_2.h"
@@ -1308,6 +1309,13 @@ int iso_node_new_symlink(char *name, char *dest, IsoSymlink **link)
     new->node.name = name;
     new->dest = dest;
     new->node.mode = S_IFLNK;
+
+#ifdef Libisofs_hardlink_matcheR
+    new->fs_id = 0;
+    new->st_dev = 0;
+    new->st_ino = 0;
+#endif
+
     *link = new;
     return ISO_SUCCESS;
 }
@@ -1340,6 +1348,12 @@ int iso_node_new_special(char *name, mode_t mode, dev_t dev,
     new->node.mode = mode;
     new->dev = dev;
 
+#ifdef Libisofs_hardlink_matcheR
+    new->fs_id = 0;
+    new->st_dev = 0;
+    new->st_ino = 0;
+#endif
+
     *special = new;
     return ISO_SUCCESS;
 }
@@ -2239,3 +2253,325 @@ int iso_node_zf_by_magic(IsoNode *node, int flag)
     return total_ret;
 }
 
+
+int iso_px_ino_xinfo_func(void *data, int flag)
+{
+    if (flag == 1) {
+        free(data);
+    }
+    return 1;
+}
+
+
+/*
+ * @param flag
+ *     bit0= do only retrieve id if node is in imported ISO image
+ *           or has an explicit xinfo inode number
+ * @return
+ *     1= reply is valid from stream, 2= reply is valid from xinfo
+ *     0= no id available,           <0= error
+ *     (fs_id, dev_id, ino_id) will be (0,0,0) in case of return <= 0
+ */
+int iso_node_get_id(IsoNode *node, unsigned int *fs_id, dev_t *dev_id,
+                    ino_t *ino_id, int flag)
+{
+    int ret;
+    IsoFile *file;
+    IsoSymlink *symlink;
+    IsoSpecial *special;
+    void *xipt;
+    
+    ret = iso_node_get_xinfo(node, iso_px_ino_xinfo_func, &xipt);
+    if (ret < 0)
+        goto no_id;
+    if (ret == 1) {
+        *fs_id = ISO_IMAGE_FS_ID;
+        *dev_id = 0;
+        *ino_id = *((ino_t *) xipt);
+        return 2;
+    }
+
+    if (node->type == LIBISO_FILE) {
+        file= (IsoFile *) node;
+        iso_stream_get_id(file->stream, fs_id, dev_id, ino_id);
+        if (*fs_id != ISO_IMAGE_FS_ID && (flag & 1)) {
+            ret = 0;
+            goto no_id;
+        }
+        return 1;
+
+#ifdef Libisofs_hardlink_matcheR
+
+    } else if (node->type == LIBISO_SYMLINK) {
+        symlink = (IsoSymlink *) node;
+        if (symlink->fs_id != ISO_IMAGE_FS_ID && (flag & 1)) {
+            ret = 0;
+            goto no_id;
+        }
+        *fs_id = symlink->fs_id;
+        *dev_id = symlink->st_dev;
+        *ino_id = symlink->st_ino;
+        return 1;
+
+    } else if (node->type == LIBISO_SPECIAL) {
+        special = (IsoSpecial *) node;
+        if (special->fs_id != ISO_IMAGE_FS_ID && (flag & 1)) {
+            ret = 0;
+            goto no_id;
+        }
+        *fs_id = special->fs_id;
+        *dev_id = special->st_dev;
+        *ino_id = special->st_ino;
+        return 1;
+
+#endif
+
+    }
+
+    ret = 0;
+no_id:;
+    *fs_id = 0;
+    *dev_id = 0;
+    *ino_id = 0;
+    return ret;
+}
+
+
+static
+int iso_node_set_ino_xinfo(IsoNode *node, ino_t ino, int flag)
+{
+    int ret;
+    void *xipt;
+
+    if (flag & 1) {
+        ret = iso_node_remove_xinfo(node, iso_px_ino_xinfo_func);
+        if (ret < 0)
+            return ret;
+    }
+    xipt = calloc(1, sizeof(ino_t));
+    if (xipt == NULL)
+        return ISO_OUT_OF_MEM;
+    memcpy(xipt, &ino, sizeof(ino_t));
+    ret = iso_node_add_xinfo(node, iso_px_ino_xinfo_func, xipt);
+    return ret;
+}
+
+
+int iso_node_set_ino(IsoNode *node, ino_t ino, int flag)
+{
+    int ret;
+    IsoFile *file;
+    IsoSymlink *symlink;
+    IsoSpecial *special;
+    void *xipt;
+    
+    ret = iso_node_get_xinfo(node, iso_px_ino_xinfo_func, &xipt);
+    if (ret < 0)
+        return ret;
+    if (ret == 1) {
+        ret = iso_node_set_ino_xinfo(node, ino, 1);
+        if (ret < 0)
+            return ret;
+        return 2;
+    }
+    if (node->type == LIBISO_FILE) {
+        file= (IsoFile *) node;
+        ret = iso_stream_set_image_ino(file->stream, ino, 0);
+        if (ret < 0 || ret == 1)
+            return ret;
+
+#ifdef Libisofs_hardlink_matcheR
+
+    } else if (node->type == LIBISO_SYMLINK) {
+        symlink = (IsoSymlink *) node;
+        if (symlink->fs_id == ISO_IMAGE_FS_ID) {
+            symlink->st_ino = ino;
+            return 1;
+        }
+
+    } else if (node->type == LIBISO_SPECIAL) {
+        special = (IsoSpecial *) node;
+        if (special->fs_id == ISO_IMAGE_FS_ID) {
+            special->st_ino = ino;
+            return 1;
+        }
+
+#endif
+
+    }
+    ret = iso_node_set_ino_xinfo(node, ino, 0);
+    if (ret < 0)
+        return ret;
+    return 2;
+}
+
+
+int iso_node_set_unique_id(IsoNode *node, IsoImage *image, int flag)
+{
+    int ret;
+    ino_t ino;
+
+    ino = img_give_ino_number(image, 0);
+    ret = iso_node_set_ino(node, ino, 0);
+    return ret;
+}
+
+/*
+ * Note to programmers: It is crucial not to break the following constraints.
+ * Anti-symmetry: cmp(X,Y) == - cmp(Y,X)
+ * Transitivity : if cmp(A,B) < 0 && cmp(B,C) < 0 then cmp(A,C) < 0
+ *                if cmp(A,B) == 0 && cmp(B,C) == 0 then cmp(A,C) == 0
+ * A big transitivity hazard are tests which do not apply to some nodes.
+ * In this case for any A that is applicable and any B that is not applicable
+ * the comparison must have the same non-zero result. I.e. a pair of applicable
+ * and non-applicable node must return that non-zero result before the test
+ * for a pair of applicable nodes would happen.
+ * 
+ * @param flag
+ *     bit0= compare stat properties and attributes 
+ *     bit1= treat all nodes with image ino == 0 as unique
+ */
+int iso_node_cmp_flag(IsoNode *n1, IsoNode *n2, int flag)
+{
+    int ret1, ret2;
+    unsigned int fs_id1, fs_id2;
+    dev_t dev_id1, dev_id2;
+    ino_t ino_id1, ino_id2;
+    IsoFile *f1 = NULL, *f2 = NULL;
+    IsoSymlink *l1 = NULL, *l2 = NULL;
+    IsoSpecial *s1 = NULL, *s2 = NULL;
+    void *x1, *x2;
+
+    if (n1 == n2)
+        return 0;
+    if (n1->type != n2->type)
+        return (n1->type < n2->type ? -1 : 1);
+
+    /* Imported or explicite ISO image node id has priority */
+    ret1 = (iso_node_get_id(n1, &fs_id1, &dev_id1, &ino_id1, 1) > 0);
+    ret2 = (iso_node_get_id(n2, &fs_id2, &dev_id2, &ino_id2, 1) > 0);
+    if (ret1 != ret2)
+        return (ret1 < ret2 ? -1 : 1);
+    if (ret1) {
+        /* fs_id and dev_id do not matter here.
+           Both nodes have explicit inode numbers of the emerging image.
+         */
+        if (ino_id1 != ino_id2)
+            return (ino_id1 < ino_id2 ? -1 : 1);
+        if (ino_id1 == 0) /* Image ino 0 is always unique */
+            return (n1 < n2 ? -1 : 1);
+        goto image_inode_match;
+    }
+
+    if (n1->type == LIBISO_FILE) {
+
+        f1 = (IsoFile *) n1;
+        f2 = (IsoFile *) n2;
+        ret1 = iso_stream_cmp_ino(f1->stream, f2->stream, 0);
+        if (ret1)
+            return ret1;
+        goto inode_match;
+
+#ifdef Libisofs_hardlink_matcheR
+
+    } else if (n1->type == LIBISO_SYMLINK) {
+
+        l1 = (IsoSymlink *) n1;
+        l2 = (IsoSymlink *) n2;
+        fs_id1 = l1->fs_id;
+        dev_id1 = l1->st_dev;
+        ino_id1 = l1->st_ino;
+        fs_id2 = l2->fs_id;
+        dev_id2 = l2->st_dev;
+        ino_id2 = l2->st_ino;
+
+    } else if (n1->type == LIBISO_SPECIAL) {
+
+        s1 = (IsoSpecial *) n1;
+        s2 = (IsoSpecial *) n2;
+        fs_id1 = s1->fs_id;
+        dev_id1 = s1->st_dev;
+        ino_id1 = s1->st_ino;
+        fs_id2 = s2->fs_id;
+        dev_id2 = s2->st_dev;
+        ino_id2 = s2->st_ino;
+
+#endif /* Libisofs_hardlink_matcheR */
+
+    } else {
+        return (n1 < n2 ? -1 : 1); /* case n1 == n2 is handled above */
+    }
+    if (fs_id1 != fs_id2)
+        return (fs_id1 < fs_id2 ? -1 : 1);
+    if (dev_id1 != dev_id2)
+        return (dev_id1 < dev_id2 ? -1 : 1);
+    if (ino_id1 != ino_id2)
+        return (ino_id1 < ino_id2 ? -1 : 1);
+    if (fs_id1 == 0 && dev_id1 == 0 && ino_id1 == 0)
+        return (n1 < n2 ? -1 : 1);
+
+inode_match:;
+
+    if (flag & 2) {
+        /* What comes here has no predefined image ino resp. image_ino == 0 .
+           Regard this as not equal.
+        */
+        return (n1 < n2 ? -1 : 1);
+    }
+
+image_inode_match:;
+
+    if (!(flag & 1))
+        return 0;
+    if (n1->type == LIBISO_SYMLINK) {
+        l1 = (IsoSymlink *) n1;
+        l2 = (IsoSymlink *) n2;
+        ret1 = strcmp(l1->dest, l2->dest);
+        if (ret1)
+            return ret1;
+    } else if (n1->type == LIBISO_SPECIAL) {
+        s1 = (IsoSpecial *) n1;
+        s2 = (IsoSpecial *) n2;
+        if (s1->dev != s2->dev)
+            return (s1->dev < s2->dev ? -1 : 1);
+    }
+
+    if (n1->mode != n2->mode)
+        return (n1->mode < n2->mode ? -1 : 1);
+    if (n1->uid != n2->uid)
+        return (n1->uid < n2->uid ? -1 : 1);
+    if (n1->gid != n2->gid)
+        return (n1->gid < n2->gid ? -1 : 1);
+    if (n1->atime != n2->atime)
+        return (n1->atime < n2->atime ? -1 : 1);
+    if (n1->mtime != n2->mtime)
+        return (n1->mtime < n2->mtime ? -1 : 1);
+    if (n1->ctime != n2->ctime)
+        return (n1->ctime < n2->ctime ? -1 : 1);
+
+    /* Compare xinfo */
+    /* :( cannot compare general xinfo because data length is not known :( */
+
+    /* compare aa_string */
+    ret1 = iso_node_get_xinfo(n1, aaip_xinfo_func, &x1);
+    ret2 = iso_node_get_xinfo(n2, aaip_xinfo_func, &x2);
+    if (ret1 != ret2)
+        return (ret1 < ret2 ? -1 : 1);
+    if (ret1 == 1) {
+        ret1 = aaip_count_bytes((unsigned char *) x1, 0);
+        ret2 = aaip_count_bytes((unsigned char *) x2, 0);
+        if (ret1 != ret2)
+            return (ret1 < ret2 ? -1 : 1);
+        ret1 = memcmp(x1, x2, ret1);
+        if (ret1)
+            return ret1;
+    }
+
+    return 0;
+}
+
+/* API */
+int iso_node_cmp_ino(IsoNode *n1, IsoNode *n2, int flag)
+{
+    return iso_node_cmp_flag(n1, n2, 1);
+}
